<template>
	<div class="feedback-smile-form" :style='widgetWrapperStyle'>
		<div class="form"  @mouseenter="hover(); updateSmileList()" @mouseleave='unhover'>
			<div
				v-for="smile in smilesArrReact"
				@mousedown="mouseDown(smile)"
				@mouseup="mouseUp(smile)"
				:key="'react'+smile"
				@mouseleave="mouseOut(smile)"
				class="smile"
				ondragstart="return false;"
				ondrop="return false;"
			>
				<img :src="api + '/static/widgets/Reactions/smile/' + smile + '.png'" alt="" />
			</div>
		</div>

		<div class="form-out">
			<div v-for="smile in smilesArr" class="smileOut" :key="'reactOut'+smile" >
				<div v-if="!outputReactionsMy[smile] || outputReactions[smile]>outputReactionsMy[smile]">
					<div :key='smile + i' v-for="i in outputReactions[smile]" class="smile smileOutput">
						<img :src="api + '/static/widgets/Reactions/smile/' + smile + '.png'" alt="" >
					</div>
				</div>
				<div :key='smile+"reactOutMy" + i' v-for="i in outputReactionsMy[smile]" class="smile smileOutput mySmile">
					<img :src="api + '/static/widgets/Reactions/smile/' + smile + '.png'" alt="" >
				</div>
			</div>
		</div>
	</div>
</template>

<script>
import {positionMixin, widgetMixin} from '../helpers/widgets.mjs'

export default {
	mixins: [widgetMixin, positionMixin],

	props: {
		contentItem: {
			type: Object,
			default: () => {},
		},
	},

	data() {
		return {
			updateInterval: null,
			fakeTimeout: null,
			reactions: {},
			outputReactions: {},
			outputReactionsMy: {},
			outputReactionsTimeouts: {},
			fakeReactions: [],
			smilesArr: [
				'red-heart',
				'accordion',
				'alien',
				'alien-monster',
				'anatomical-heart',
				'artist-palette',
				'baby-bottle',
				'black-heart',
				'blue-heart',
				'broken-heart',
				'candle',
				'cherries',
				'clapping-hands',
				'direct-hit',
				'dodo',
				'duck',
				'ewe',
				'eyes',
				'fire',
				'flushed-face',
				'full-moon-face',
				'gem-stone',
				'glowing-star',
				'goblin',
				'gorilla',
				'hand-with-index-finger-and-thumb-crossed',
				'heart-on-fire',
				'high-voltage',
				'hot-dog',
				'kiss-mark',
				'lollipop',
				'microbe',
				'moai',
				'monkey',
				'mushroom',
				'neutral-face',
				'new-moon-face',
				'ok-hand',
				'pile-of-poo',
				'rock',
				'rocket',
				'sign-of-the-horns',
				'snail',
				'sparkling-heart',
				'sports-medal',
				'star-struck',
				'sweat-droplets',
				'thumbs-down',
				'thumbs-up',
				'waving-hand',
				'white-heart',
				'zebra',
				'zzz'],
			smilesArrReact: [],
			smilesCnt: 5,
			clickTimeout: null,
			clickSended: null,
		}
	},

	created() {
		this.updateSmileList()
	},

	watch: {
		reactions(newVal, oldVal) {
			// eslint-disable-next-line guard-for-in
			for (const key in newVal) {
				const difference = newVal[key] - oldVal[key]

				for (let i = 0; i < difference; i++) {
					setTimeout(
						() => {
							this.addReaction(key)
						},
						i * (6000 / difference),
					)
				}
			}
		},
	},


	mounted() {
		this.updateInterval = setInterval(this.getData, 6000)

		this.addFakeReactions()
	},

	beforeUnmount() {
		clearInterval(this.updateInterval)
		clearTimeout(this.fakeTimeout)
		clearInterval(this.clickTimeout)
	},

	methods: {
		mouseDown(value) {
			const callback = () => {
				this.clickSended = true
				this.submit(value)
			}

			this.clickTimeout = setInterval(callback, 150)
			callback()
		},

		mouseUp(value) {
			if (!this.clickSended) {
				this.submit(value)
			}
			clearInterval(this.clickTimeout)
			this.clickSended = false
		},

		updateSmileList() {
			const shuffled = this.smilesArr.slice(1).sort(() => 0.5 - Math.random())
			this.smilesArrReact = [this.smilesArr[0], ...shuffled.slice(0, this.smilesCnt - 1)]
		},

		mouseOut() {
			clearInterval(this.clickTimeout)
			this.clickSended = false
		},

		submit(value) {
			this.outputReactionsMy = {
				...this.outputReactionsMy,
				[value]: (this.outputReactionsMy[value] ?? 0) + 1,
			}

			this.act('react', {smile: value}).then((data) => {
				const values = data.data
				this.setData(values)
			})
		},

		addFakeReactions() {
			const randomSmile = this.smilesArr[Math.floor(Math.random() * this.smilesArr.length)]
			const number = Math.floor(Math.random() * 10 + 6)

			for (let i = 0; i < number; i++) {
				setTimeout(() => {
					this.addReaction(randomSmile)
				}, 150 * i)
			}

			this.fakeTimeout = setTimeout(this.addFakeReactions, Math.random() * 10000)
		},

		addReaction(type) {
			this.outputReactions = {
				...this.outputReactions,
				[type]: (this.outputReactions[type] ?? 0) + 1,
			}

			clearTimeout(this.outputReactionsTimeouts[type])
			this.outputReactionsTimeouts[type] = setTimeout(()=>{
				this.$set(this.outputReactions, type, 0)
				this.$set(this.outputReactionsMy, type, 0)
			}, 2200)
		},

		hover() {
			// Хак для каментов
			document.querySelector('.comments-widget')?.classList.add('hovered')
		},

		unhover() {
			// Хак для каментов
			document.querySelector('.comments-widget')?.classList.remove('hovered')
		},
	},
}
</script>

<style scoped>
.form {
	display: flex;
	align-items: center;

	position: absolute;
	bottom: .72em;
	right: .5em;

	gap: 0;

	opacity: .5;
	transition: opacity .2s ease, gap .2s ease;
	pointer-events: all;
	flex-direction: column-reverse;
	z-index:2;

	width: 1.6em;
}

.form:hover {
	opacity: 1;

	gap: 1.5em;
}

.form:hover .smile {
	opacity: 1;
}

.form:not(:hover) .smile:not(:first-child) {
	opacity: 0;
}

.form-out {
	position: absolute;
	bottom: 2em;
	right: 2em;
	z-index: 1;
}

.smile {
	position: relative;
	cursor: pointer;

	transition: opacity .2s ease;

	font-size: 1.2em;

	min-height: 0;
	flex-basis: 0;

	display: flex;
	flex-direction: column-reverse;
}

.smile >img{
	position: relative;
	z-index: 5;
	transition: transform .2s ease;
}

.smile:hover img {
	transform: scale(1.2);
}

.smile img {
	width: .8em;
	height: .8em;
}

.smileOutput {
	position: absolute;
	pointer-events: none;
}

.smileOutput:nth-child(4n+1) {
	animation: 2.2s ease-in-out fadeDown1;
	animation-fill-mode: forwards;
}

.smileOutput:nth-child(4n+2) {
	animation: 2.2s ease-in-out fadeDown2;
	animation-fill-mode: forwards;
}

.smileOutput:nth-child(4n+3) {
	animation: 2.2s ease-in-out fadeDown3;
	animation-fill-mode: forwards;
}

.smileOutput:nth-child(4n+4) {
	animation: 2.2s ease-in-out fadeDown1;
	animation-fill-mode: forwards;
}

.smileOutput.mySmile{
	animation: 2.2s ease-in-out fadeDown4;
	animation-fill-mode: forwards;
}

.smileOutput.mySmile img{
	transform: scale(1.5);
}

@keyframes fadeDown1 {
	from {
		top: 0;
		left: 0;
		opacity: 0;
		transform: rotate(-30deg);
	}
	15% {
		opacity: 1;
	}
	33% {
		left: -1.2em;
	}
	50% {
		transform: rotate(10deg);
	}
	66% {
		left: 0;
	}
	to {
		top: -10em;
		left: -2.5em;
		opacity: 0;
		visibility: hidden;
		transform: rotate(30deg);
	}
}

@keyframes fadeDown2 {
	from {
		top: 0;
		left: .5em;
		opacity: 0;
		transform: rotate(50deg);
	}
	15% {
		opacity: 1;
	}
	33% {
		left: -1.2em;
	}
	50% {
		transform: rotate(-10deg);
	}
	66% {
		left: -2.5em;
	}
	to {
		top: -10em;
		left: -1em;
		opacity: 0;
		visibility: hidden;
		transform: rotate(30deg);
	}
}

@keyframes fadeDown3 {
	from {
		top: 0;
		left: -1em;
		opacity: 0;
		transform: rotate(-10deg);
	}
	15% {
		opacity: 1;
	}
	33% {
		left: -.5em;
	}
	50% {
		transform: rotate(10deg);
	}
	66% {
		left: -3em;
	}
	to {
		top: -10em;
		left: 1.5em;
		opacity: 0;
		visibility: hidden;
		transform: rotate(30deg);
	}
}
@keyframes fadeDown4 {
	from {
		top: 0;
		left: -1em;
		opacity: 0;
		transform: rotate(-50deg);
	}
	15% {
		opacity: 1;
	}
	33% {
		left: -2em;
	}
	50% {
		transform: rotate(20deg);
	}
	66% {
		left: -3em;
	}
	to {
		top: -10em;
		left: -4em;
		opacity: 0;
		visibility: hidden;
		transform: rotate(0deg);
	}
}
</style>
